/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.core.archive.compound;

import java.io.IOException;
import java.util.HashMap;

class BlockManager
{

	static final int DEFAULT_CACHE_BLOCKS = 64;
	BlockManagerEventAdapter eventAdapter;

	int cacheSize;
	HashMap blocks;
	Block firstBlock;
	Block lastBlock;

	BlockManager( BlockManagerEventAdapter adapter )
	{
		this.eventAdapter = adapter;
		this.cacheSize = DEFAULT_CACHE_BLOCKS;
		blocks = new HashMap( cacheSize );
		firstBlock = null;
		lastBlock = null;
	}

	int getCacheSize( )
	{
		return cacheSize;
	}

	void setCacheSize( int cacheSize )
	{
		if ( cacheSize <= 2 )
		{
			cacheSize = DEFAULT_CACHE_BLOCKS;
		}
		this.cacheSize = cacheSize;
	}

	void close( )
	{
		blocks.clear( );
		firstBlock = null;
		lastBlock = null;
	}

	void flush( ) throws IOException
	{
		if ( eventAdapter != null )
		{
			Block block = firstBlock;
			while ( block != null )
			{
				eventAdapter.flush( block );
				block = block.next;
			}
		}
	}

	Block getBlock( int id ) throws IOException
	{
		if ( firstBlock == null )
		{
			Block block = new Block( );
			block.id = id;
			block.prev = null;
			block.next = null;
			if ( eventAdapter != null )
			{
				eventAdapter.refresh( block );
			}
			firstBlock = lastBlock = block;
			return block;
		}

		if ( id == firstBlock.id )
		{
			return firstBlock;
		}

		Block block = (Block) blocks.get( new Integer( id ) );
		if ( block == null )
		{
			if ( blocks.size( ) >= cacheSize )
			{
				block = lastBlock;
				lastBlock = lastBlock.prev;
				lastBlock.next = null;

				blocks.remove( new Integer( block.id ) );
				if ( eventAdapter != null )
				{
					eventAdapter.flush( block );
				}
				block.reset( );
			}
			else
			{
				block = new Block( );
			}
			block.id = id;
			if ( eventAdapter != null )
			{

				eventAdapter.refresh( block );
			}
			blocks.put( new Integer( id ), block );
		}
		else
		{
			block.prev.next = block.next;
			if ( lastBlock != block )
			{
				block.next.prev = block.prev;
			}
			else
			{
				lastBlock = block.prev;
			}
		}
		block.prev = null;
		block.next = firstBlock;
		firstBlock.prev = block;
		firstBlock = block;
		return block;
	}
}
