/*
 *************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *  
 *************************************************************************
 */

package org.eclipse.birt.data.oda.adapter.dtp;

import org.eclipse.datatools.connectivity.oda.IParameterMetaData;
import org.eclipse.datatools.connectivity.oda.OdaException;

/**
 * ParameterMetaData is the DTP ODA adapter for the BIRT ODA driver interface IParameterMetaData.
 */
public class ParameterMetaData implements IParameterMetaData
{	

    private org.eclipse.birt.data.oda.IParameterMetaData m_birtParameterMetaData = null;

    private ParameterMetaData()
    {
        
    }
    
    /**
     * Creates a ParameterMetaData adapter for the specified object.
     * @param birtParameterMetaData		BIRT ODA IParameterMetaData interface implementation.
     */
    public ParameterMetaData( org.eclipse.birt.data.oda.IParameterMetaData birtParameterMetaData )
	{
        m_birtParameterMetaData = birtParameterMetaData;
	}

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IParameterMetaData#getParameterCount()
     */
    public int getParameterCount() throws OdaException
    {
        try
        {
            return m_birtParameterMetaData.getParameterCount();
        }
        catch( org.eclipse.birt.data.oda.OdaException e )
        {
            throw new OdaAdapterException( e );
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IParameterMetaData#getParameterMode(int)
     */
    public int getParameterMode( int param ) throws OdaException
    {
        try
        {
            return m_birtParameterMetaData.getParameterMode( param );
        }
        catch( org.eclipse.birt.data.oda.OdaException e )
        {
            throw new OdaAdapterException( e );
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IParameterMetaData#getParameterType(int)
     */
    public int getParameterType( int param ) throws OdaException
    {
        try
        {
            return m_birtParameterMetaData.getParameterType( param );
        }
        catch( org.eclipse.birt.data.oda.OdaException e )
        {
            throw new OdaAdapterException( e );
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IParameterMetaData#getParameterTypeName(int)
     */
    public String getParameterTypeName( int param ) throws OdaException
    {
        try
        {
            return m_birtParameterMetaData.getParameterTypeName( param );
        }
        catch( org.eclipse.birt.data.oda.OdaException e )
        {
            throw new OdaAdapterException( e );
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IParameterMetaData#getPrecision(int)
     */
    public int getPrecision( int param ) throws OdaException
    {
        try
        {
            return m_birtParameterMetaData.getPrecision( param );
        }
        catch( org.eclipse.birt.data.oda.OdaException e )
        {
            throw new OdaAdapterException( e );
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IParameterMetaData#getScale(int)
     */
    public int getScale( int param ) throws OdaException
    {
        try
        {
            return m_birtParameterMetaData.getScale( param );
        }
        catch( org.eclipse.birt.data.oda.OdaException e )
        {
            throw new OdaAdapterException( e );
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IParameterMetaData#isNullable(int)
     */
    public int isNullable( int param ) throws OdaException
    {
        try
        {
            return m_birtParameterMetaData.isNullable( param );
        }
        catch( org.eclipse.birt.data.oda.OdaException e )
        {
            throw new OdaAdapterException( e );
        }
    }
	
}
