/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.core.model.views.outline;

import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.SlotHandle;

/**
 * Contains the top-level slotHandle like body, masterpages,etc.
 */

public class ReportElementModel
{

	private SlotHandle slotHandle = null;

	/**
	 * Constructor
	 * 
	 * @param slotHandle
	 *  
	 */
	public ReportElementModel( SlotHandle slotHandle )
	{
		this.slotHandle = slotHandle;
	}

	/**
	 * Constructor. Sets the elementHandle and slotId
	 * 
	 * @param ele
	 *            the elementHandle to be set
	 * @param id
	 *            the slotId to be set
	 */

	public ReportElementModel( DesignElementHandle ele, int id )
	{
		slotHandle = ele.getSlot( id );
	}

	/**
	 * Gets the handle of the slot
	 * 
	 * @return Returns the slot handle
	 */
	public SlotHandle getSlotHandle( )
	{
		return slotHandle;
	}

	/**
	 * Gets the handle of the element
	 * 
	 * @return Returns the handle of the element
	 */

	public DesignElementHandle getElementHandle( )
	{
		return slotHandle.getElementHandle( );
	}

	/**
	 * Gets the slot id
	 * 
	 * @return Returns the slot id
	 */

	public int getSlotId( )
	{
		return slotHandle.getSlotID( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals( Object obj )
	{
		if ( obj instanceof ReportElementModel )
		{
			return ( obj == this )
					|| ( (ReportElementModel) obj ).slotHandle == slotHandle;
		}
		else if (obj instanceof SlotHandle)
		{
			return slotHandle.equals(obj);
		}
		return false;
	}
	
	public int hashCode(){
		return slotHandle.hashCode( );
	}
}