/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation. All rights reserved. This program and
 * the accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Actuate Corporation - initial API and implementation
 ******************************************************************************/

package org.eclipse.birt.report.designer.data.ui.dataset;

import java.util.Iterator;

import org.eclipse.birt.core.exception.BirtException;
import org.eclipse.birt.report.designer.data.ui.property.AbstractDescriptionPropertyPage;
import org.eclipse.birt.report.designer.data.ui.util.DataSetProvider;
import org.eclipse.birt.report.designer.internal.ui.util.ExceptionHandler;
import org.eclipse.birt.report.designer.internal.ui.util.Utility;
import org.eclipse.birt.report.designer.nls.Messages;
import org.eclipse.birt.report.model.api.ColumnHintHandle;
import org.eclipse.birt.report.model.api.DataSetHandle;
import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.PropertyHandle;
import org.eclipse.birt.report.model.api.activity.NotificationEvent;
import org.eclipse.birt.report.model.api.activity.SemanticException;
import org.eclipse.birt.report.model.api.core.Listener;
import org.eclipse.birt.report.model.api.elements.structures.ColumnHint;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

/**
 * Property page to edit output columns for all oda data sets.
 */
public final class OutputColumnsPage extends AbstractDescriptionPropertyPage
		implements
			Listener
{

	private transient TableViewer outputColumnsViewer = null;
	// to indicate the status of dataset handle
	private transient boolean modelChanged = true;

	/**
	 * The constructor.
	 */
	public OutputColumnsPage( )
	{
		super( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.IPropertyPage#createPageControl(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents( Composite parent )
	{
		outputColumnsViewer = new TableViewer( parent, SWT.FULL_SELECTION );
		outputColumnsViewer.getTable( ).setHeaderVisible( true );
		outputColumnsViewer.getTable( ).setLinesVisible( true );

		TableColumn column = new TableColumn( outputColumnsViewer.getTable( ),
				SWT.LEFT );
		column.setText( Messages.getString( "dataset.editor.title.name" ) ); //$NON-NLS-1$
		column.setWidth( 100 );
		column = new TableColumn( outputColumnsViewer.getTable( ), SWT.LEFT );
		column.setText( Messages.getString( "dataset.editor.title.type" ) ); //$NON-NLS-1$
		column.setWidth( 100 );
        column = new TableColumn( outputColumnsViewer.getTable( ), SWT.LEFT );
        column.setText( Messages.getString( "dataset.editor.title.alias" ) ); //$NON-NLS-1$
        column.setWidth( 100 );
		column = new TableColumn( outputColumnsViewer.getTable( ), SWT.LEFT );
		column.setText( Messages.getString( "dataset.editor.title.displayName" ) ); //$NON-NLS-1$
		column.setWidth( 100 );
  
		//TODO  Add support to helpText after release 1
		//      column = new TableColumn( outputColumnsViewer.getTable( ), SWT.LEFT );
		//      column.setText( Messages.getString( "dataset.editor.title.helpText" ) ); //$NON-NLS-1$
		//      column.setWidth( 100 );

		outputColumnsViewer.setContentProvider( new OutputColumnsContentProvider( ) );
		outputColumnsViewer.setLabelProvider( new OutputColumnsLabelProvider( ) );
		setupEditors( );

		( (DataSetHandle) getContainer( ).getModel( ) ).addListener( this );
		return outputColumnsViewer.getControl( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.IPropertyPage#pageActivated()
	 */
	public void pageActivated( )
	{
		if ( modelChanged )
		{
			updateOutputColumns( );
			modelChanged = false;
		}
		getContainer( ).setMessage( Messages.getString( "dataset.editor.outputColumns" ), //$NON-NLS-1$
				IMessageProvider.NONE );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#canLeave()
	 */
	public boolean canLeave( )
	{
		if ( isValid( ) )
		{
			saveOutputColumns( );
			return super.canLeave( );
		}
		else
			return false;
	}
	

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#performOk()
	 */
	public boolean performOk( )
	{
		if ( isValid( ) )
		{
			saveOutputColumns( );
			( (DataSetHandle) getContainer( ).getModel( ) ).removeListener( this );
			return super.performOk( );
		}
		else
			return false;
	}
	
    /* (non-Javadoc)
     * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractDescriptionPropertyPage#getPageDescription()
     */
    public String getPageDescription()
    {
        return Messages.getString("OutputColumnsPage.description"); //$NON-NLS-1$
    }

	/* (non-Javadoc)
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#getToolTip()
	 */
	public String getToolTip( )
	{
		return Messages.getString("OutputColumnPage.OutputColumns.Tooltip"); //$NON-NLS-1$
	}

	/**
	 * setup cell editors
	 *
	 */
	private void setupEditors( )
	{
		CellEditor[] editors = new CellEditor[5];
		editors[0] = new TextCellEditor( outputColumnsViewer.getTable( ) );
		editors[2] = new TextCellEditor( outputColumnsViewer.getTable( ) );
		editors[3] = new TextCellEditor( outputColumnsViewer.getTable( ) );
		editors[4] = new TextCellEditor( outputColumnsViewer.getTable( ) );
		outputColumnsViewer.setColumnProperties( new String[]{
				"name", "dataTypeName", "alias", "realDisplayName", "helpText" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		} );
		outputColumnsViewer.setCellEditors( editors );
		outputColumnsViewer.setCellModifier( new ICellModifier( ) {

			public boolean canModify( Object element, String property )
			{
				if ( "name".equals( property ) ) //$NON-NLS-1$
				{
					return false;
				}
				return true;
			}

			public Object getValue( Object element, String property )
			{
				String value = null;
				try
				{
					value = (String) Utility.getProperty( element, property );
				}
				catch ( Exception e )
				{
					ExceptionHandler.handle( e );
				}

				return ( ( value == null ) ? "" : value ); //$NON-NLS-1$
			}

			public void modify( Object element, String property, Object value )
			{
				DataSetViewData item = (DataSetViewData) ( ( (TableItem) element ).getData( ) );
				try
				{
					Utility.setProperty( item, property, value );
				}
				catch ( Exception e )
				{
					ExceptionHandler.handle( e );
				}
				outputColumnsViewer.update( item, null );
				updateMessage( );
			}

		} );
	}

    /**
	 * update output columns page
	 * 
	 */
	protected final void updateOutputColumns( )
	{
		clearOutputColumns( );
		try
		{
			populateOutputColums( );
		}
		catch ( BirtException e )
		{
			ExceptionHandler.handle( e );
		}
	}
	
	/**
	 * populate output columns
	 * @throws BirtException
	 */
	private void populateOutputColums( ) throws BirtException
	{
		DataSetViewData[] viewDatas = DataSetProvider.getCurrentInstance( )
				.populateAllOutputColumns( ( (DataSetEditor) getContainer( ) ).getHandle( ) );
		if ( viewDatas == null )
		{
			viewDatas = ( (DataSetEditor) getContainer( ) ).getCurrentItemModel( false,
					false );
		}
		outputColumnsViewer.setInput( viewDatas );
	}
	
	/**
	 * clear output columns table viewer
	 *
	 */
	protected final void clearOutputColumns( )
	{
		outputColumnsViewer.getTable( ).removeAll( );
	}

	/**
	 * 
	 *
	 */
	private void saveOutputColumns( )
	{
		PropertyHandle handle = ( (DataSetHandle) getContainer( ).getModel( ) ).getPropertyHandle( DataSetHandle.COLUMN_HINTS_PROP );
		if ( outputColumnsViewer != null
				&& outputColumnsViewer.getInput( ) != null )
		{
			//Iterate through all the cached model items
			DataSetViewData[] items = (DataSetViewData[]) outputColumnsViewer.getInput( );
			String columnName = null;
			boolean found = false;
			for ( int n = 0; n < items.length; n++ )
			{
				columnName = items[n].getName( );
				//Find this column name in the handle
				found = false;
				Iterator iter = handle.iterator( );
				if ( iter != null )
				{
					while ( iter.hasNext( ) && !found)
					{
						ColumnHintHandle hint = (ColumnHintHandle) iter.next( );
						if ( hint.getColumnName( ).equals( columnName ) )
						{
							found = true;
							//Update the display name
                            if(items[n].getRealDisplayName() == null || items[n].getRealDisplayName().trim().length() == 0)
                            {
                                hint.setDisplayName(null);
                            }
                            else
                            {
                                hint.setDisplayName( items[n].getRealDisplayName( ) );
                            }
                            if(items[n].getAlias() == null || items[n].getAlias().trim().length() ==  0)
                            {
                                hint.setAlias(null);
                            }
                            else
                            {
                                hint.setAlias(items[n].getAlias());
                            }
                            if(items[n].getHelpText() == null || items[n].getHelpText().trim().length() ==  0)
                            {
                                hint.setHelpText(null);
                            }
                            else
                            {
                                hint.setHelpText(items[n].getHelpText());
                            }
						}
					}
				}
				//If not found then create a new item and add it in
				if ( !found && isColumnHintRequired(items[n]))
				{
					ColumnHint hint = new ColumnHint( );
					hint.setProperty( ColumnHint.COLUMN_NAME_MEMBER, columnName );
                    if(items[n].getRealDisplayName() != null && items[n].getRealDisplayName().trim().length() > 0)
                    {
                        hint.setProperty( ColumnHint.DISPLAY_NAME_MEMBER,
                                items[n].getRealDisplayName( ) );
                    }
                    if(items[n].getAlias() != null && items[n].getAlias().trim().length() > 0)
                    {
                        hint.setProperty(ColumnHint.ALIAS_MEMBER, items[n].getAlias());
                    }
                    if(items[n].getHelpText() != null && items[n].getHelpText().trim().length() > 0)
                    {
                        hint.setProperty(ColumnHint.HELP_TEXT_MEMBER, items[n].getHelpText());
                        
                    }
					try
					{
						handle.addItem( hint );
					}
					catch ( SemanticException e )
					{
						ExceptionHandler.handle( e );
					}
				}
			}
		}
	}
    
	/**
	 * validates that the new name/alias from a column hint or a new custom
	 * column doesn't conflict with existing column names or aliases
	 * 
	 */
	private boolean isValid( )
	{
		boolean validate = true;
		String newColumnNameOrAlias;
		DataSetViewData[] items = null;

		if ( outputColumnsViewer == null )
		{
			items = DataSetProvider.getCurrentInstance( )
					.getColumns( ( (DataSetEditor) getContainer( ) ).getHandle( ),
							false,
							true,
							true );
		}
		else
		{
			items = (DataSetViewData[]) outputColumnsViewer.getInput( );
		}
		for ( int i = 0; items != null && i < items.length && validate; i++ )
		{
			newColumnNameOrAlias = items[i].getAlias( );
			if ( newColumnNameOrAlias != null
					&& newColumnNameOrAlias.length( ) > 0 )
			{
				for ( int n = 0; n < items.length; n++ )
				{
					if ( i == n )
						continue;
					if ( ( items[n].getName( ) != null && items[n].getName( )
							.equals( newColumnNameOrAlias ) )
							|| ( items[n].getAlias( ) != null && items[n].getAlias( )
									.equals( newColumnNameOrAlias ) ) )
					{
						validate = false;
						getContainer( ).setMessage( Messages.getFormattedString( "dataset.editor.error.columnOrAliasNameAlreadyUsed",
								new Object[]{
										newColumnNameOrAlias,
										new Integer( n + 1 )
								} ),
								IMessageProvider.ERROR );

						break;
					}
				}
			}
		}
		return validate;
	}
	
	/**
	 * update message
	 *
	 */
	private void updateMessage( )
	{
		if ( isValid( ) )
			getContainer( ).setMessage( Messages.getString( "dataset.editor.outputColumns" ), //$NON-NLS-1$
					IMessageProvider.NONE );
	}
	
	/**
	 * 
	 * @param model
	 * @return
	 */
    private boolean isColumnHintRequired(DataSetViewData model)
    {
        return !(
                (model.getRealDisplayName() == null || model.getRealDisplayName().trim().length() == 0)
                && (model.getAlias() == null || model.getAlias().trim().length() == 0)
                && (model.getHelpText() == null || model.getHelpText().trim().length() == 0)
                );
    }

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.core.Listener#elementChanged(org.eclipse.birt.model.api.DesignElementHandle,
	 *      org.eclipse.birt.model.activity.NotificationEvent)
	 */
	public void elementChanged( DesignElementHandle focus, NotificationEvent ev )
	{
		if ( focus.equals( getContainer( ).getModel( ) ) )
		{
			modelChanged = true;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#performCancel()
	 */
	public boolean performCancel( )
	{
		if ( outputColumnsViewer != null
				&& outputColumnsViewer.getInput( ) != null )
		{
			//Update the cached list from the model
			//We have to do this because we are directly updating the column
			// names in the cached list
			DataSetProvider manager = DataSetProvider.getCurrentInstance( );
			manager.updateModel( ( (DataSetEditor) getContainer( ) ).getHandle( ),
					(DataSetViewData[]) outputColumnsViewer.getInput( ) );
		}

		( (DataSetHandle) getContainer( ).getModel( ) ).removeListener( this );
		return super.performCancel( );
	}
}

class OutputColumnsContentProvider implements IStructuredContentProvider
{

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements( Object inputElement )
	{
		return (DataSetViewData[]) inputElement;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose( )
	{

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
	 *      java.lang.Object, java.lang.Object)
	 */
	public void inputChanged( Viewer viewer, Object oldInput, Object newInput )
	{

	}
}

class OutputColumnsLabelProvider implements ITableLabelProvider
{

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
	 *      int)
	 */
	public Image getColumnImage( Object element, int columnIndex )
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
	 *      int)
	 */
	public String getColumnText( Object element, int columnIndex )
	{
		String value = null;
		DataSetViewData item = (DataSetViewData) element;
		switch ( columnIndex )
		{
			case 0 :
			{
				value = item.getName( );
				break;
			}
			case 1 :
			{
				value = item.getDataTypeName( );
				break;
			}
			case 2 :
			{
				value = item.getAlias();
				break;
			}
            case 3:
            {
                value = item.getRealDisplayName();
                break;
            }
            case 4:
            {
                value = item.getHelpText();
                break;
            }
		}
		return ((value == null)?"":value);//$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener( ILabelProviderListener listener )
	{

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose( )
	{

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object,
	 *      java.lang.String)
	 */
	public boolean isLabelProperty( Object element, String property )
	{
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener( ILabelProviderListener listener )
	{

	}

}