/*******************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.data.ui.dataset;

import org.eclipse.birt.report.designer.data.ui.util.DTPUtil;
import org.eclipse.birt.report.designer.internal.ui.util.ExceptionHandler;
import org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractPropertyPage;
import org.eclipse.birt.report.model.api.OdaDataSetHandle;
import org.eclipse.datatools.connectivity.oda.OdaException;
import org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetEditorPage;
import org.eclipse.jface.preference.IPreferencePageContainer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.dialogs.PropertyPage;

/**
 * To wrap PropertyPage to IPropertyPage
 */

public class PropertyPageWrapper extends AbstractPropertyPage
{

	private PropertyPage propertyPage = null;

	public PropertyPageWrapper( PropertyPage propertyPage )
	{
		this.propertyPage = propertyPage;
	}

	public Control createPageControl( Composite parent )
	{
		propertyPage.setContainer( (IPreferencePageContainer) getContainer( ) );
		propertyPage.createControl( parent );

		return propertyPage.getControl( );
	}

	public void pageActivated( )
	{
		if ( propertyPage instanceof DataSetEditorPage )
			( (DataSetEditorPage) propertyPage ).refresh( );

		getContainer( ).setMessage( propertyPage.getMessage( ),
				propertyPage.getMessageType( ) );
	}

	public PropertyPage getPropertyPage( )
	{
		return propertyPage;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.IPropertyPage#getToolTip()
	 */
	public String getToolTip( )
	{
		return propertyPage.getTitle( );
	}	 

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractPropertyPage#canLeave()
	 */
	public boolean canLeave( )
	{
		if ( propertyPage instanceof DataSetEditorPage )
		{
			if ( ( (DataSetEditorPage) propertyPage ).okToLeave( ) )
			{
				try
				{
					DTPUtil.getInstance( )
							.updateDataSetHandle( ( (DataSetEditorPage) propertyPage ).collectPageResponse( ),
									(OdaDataSetHandle) ( (DataSetEditor) getContainer( ) ).getModel( ),
									false );
				}
				catch ( OdaException e )
				{
					ExceptionHandler.handle( e );
				}
			}
			return ( (DataSetEditorPage) propertyPage ).okToLeave( );
		}

		return super.canLeave( );
	}

}
