/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.internal.ui.views.attributes.widget;

import org.eclipse.birt.report.designer.internal.ui.views.attributes.page.WidgetUtil;
import org.eclipse.birt.report.designer.ui.views.attributes.providers.PropertyProcessor;
import org.eclipse.birt.report.model.api.activity.SemanticException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

/**
 * FontSizePropertyDescriptor manages Font size control.
 */
public class FontSizePropertyDescriptor extends AbstractPropertyDescriptor
{

	private FontSizeBuilder builder;

	private boolean processing = false;

	/**
	 * @param propertyProcessor
	 *            the property handle
	 */
	public FontSizePropertyDescriptor( PropertyProcessor propertyProcessor )
	{
		super( propertyProcessor );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.internal.ui.views.attributes.widget.PropertyDescriptor#resetUIData()
	 */
	public void resetUIData( )
	{
		String strValue = getPropertyProcessor( ).getStringValue(
				getModelList( ) );
		boolean stateFlag = ( ( strValue == null ) == builder.getEnabled( ) );
		if ( stateFlag )
		{
			builder.setEnabled( strValue != null );
		}
		if(getPropertyProcessor( ).isReadOnly( getModelList( ) ))
		{
			builder.setEnabled(false);
		}
		builder.setFontSizeValue( strValue );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.internal.ui.views.attributes.widget.PropertyDescriptor#getControl()
	 */
	public Control getControl( )
	{
		return builder;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.extensions.IPropertyDescriptor#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public Control createControl( Composite parent )
	{
		builder = new FontSizeBuilder( parent, SWT.NONE );
		builder.addListener( SWT.Modify, new Listener( ) {

			public void handleEvent( Event event )
			{
				if ( !processing )
				{
					processing = true;
					builder.setDefaultUnit( getPropertyProcessor( )
							.getDefaultUnit( getModelList( ) ) );
					processAction( );
					processing = false;
				}
			}
		} );
		return builder;
	}

	/**
	 * Processes the save action.
	 */
	private void processAction( )
	{
		String oldValue = getPropertyProcessor( ).getStringValue(
				getModelList( ) );

		String sizeValue = builder.getFontSizeValue( );

		if ( sizeValue == null || sizeValue.length( ) == 0 )
		{
			if ( oldValue == null || oldValue.length( ) == 0 )
			{
				return;
			}
		}

		if ( sizeValue != null && sizeValue.equals( oldValue ) )
		{
			return;
		}

		try
		{
			getPropertyProcessor( ).setStringValue( sizeValue, getModelList( ) );
		}
		catch ( SemanticException e )
		{
			WidgetUtil.processError( builder.getShell( ), e );
			builder.setFontSizeValue( oldValue );
			return;
		}

		if ( sizeValue == null || sizeValue.length( ) == 0 )
		{
			String sizeString = getPropertyProcessor( ).getStringValue(
					getModelList( ) );
			builder.setFontSizeValue( sizeString );
		}
	}
}