/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.ui.views.attributes;

import java.util.List;

import org.eclipse.birt.report.designer.internal.ui.views.attributes.page.BindingPage;
import org.eclipse.birt.report.designer.internal.ui.views.attributes.page.FormPage;
import org.eclipse.birt.report.designer.internal.ui.views.attributes.page.HighlightsPage;
import org.eclipse.birt.report.designer.internal.ui.views.attributes.page.MapPage;
import org.eclipse.birt.report.designer.internal.ui.views.attributes.provider.FilterHandleProvider;
import org.eclipse.birt.report.designer.internal.ui.views.attributes.provider.GroupHandleProvider;
import org.eclipse.birt.report.designer.internal.ui.views.attributes.provider.SortingHandleProvider;
import org.eclipse.birt.report.designer.nls.Messages;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;

/**
 * Instances of TablePageGenerator take change of creating attribute page
 * correspond to DE Table element.
 */
public class TablePageGenerator extends CategoryPageGenerator
{

	/**
	 * SWT widgets presented Sorting properties of DE Table element
	 */
	private FormPage sortingPage;

	/**
	 * SWT widgets presented filter properties of DE Table element
	 */
	private FormPage filterPage;

	/**
	 * SWT widgets presented Groups properties of DE Table element
	 */
	private FormPage groupPage;

	/**
	 * SWT widgets presented binding properties of DE Table element
	 */
	private BindingPage bindingPage;

	/**
	 * SWT widgets presented highlights properties of DE Table element
	 */
	private HighlightsPage highlightsPage;

	private MapPage mapPage;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.views.attributes.IPageGenerator#createTabItems(org.eclipse.swt.widgets.TabFolder,
	 *      java.util.List)
	 */
	public void createTabItems( TabFolder tabFolder, List input )
	{
		super.createTabItems( tabFolder, input );

		TabItem tabItem;

		if ( bindingPage == null )
		{
			bindingPage = new BindingPage( tabFolder, SWT.NONE );
			tabItem = new TabItem( tabFolder, SWT.NONE );
			tabItem.setText( Messages.getString( "TablePageGenerator.TabItem.Binding" ) ); //$NON-NLS-1$
			tabItem.setControl( bindingPage );

		}
		bindingPage.setInput( input );

		if ( sortingPage == null )
		{
			sortingPage = new FormPage( tabFolder,
					FormPage.FULL_FUNCTION,
					new SortingHandleProvider( ),
					true);
			tabItem = new TabItem( tabFolder, SWT.NONE );
			tabItem.setText( Messages.getString( "TablePageGenerator.TabItem.Sorting" ) ); //$NON-NLS-1$
			tabItem.setControl( sortingPage );
		}
		sortingPage.setInput( input );

		if ( groupPage == null )
		{
			groupPage = new FormPage( tabFolder,
					FormPage.FULL_FUNCTION,
					new GroupHandleProvider( ),
					true );
			tabItem = new TabItem( tabFolder, SWT.NONE );
			tabItem.setText( Messages.getString( "TablePageGenerator.TabItem.Groups" ) ); //$NON-NLS-1$
			tabItem.setControl( groupPage );
		}
		groupPage.setInput( input );

		if ( mapPage == null )
		{
			if ( mapPage == null )
			{
				mapPage = new MapPage( tabFolder, SWT.NONE );
				tabItem = new TabItem( tabFolder, SWT.NONE );
				tabItem.setText( Messages.getString( "TablePageGenerator.TabItem.map" ) ); //$NON-NLS-1$
				tabItem.setControl( mapPage );
			}
		}
		mapPage.setInput( input );

		if ( highlightsPage == null )
		{
			if ( highlightsPage == null )
			{
				highlightsPage = new HighlightsPage( tabFolder, SWT.NONE );
				tabItem = new TabItem( tabFolder, SWT.NONE );
				tabItem.setText( Messages.getString( "TablePageGenerator.TabItem.Highlights" ) ); //$NON-NLS-1$
				tabItem.setControl( highlightsPage );
			}
		}
		highlightsPage.setInput( input );

		if ( filterPage == null )
		{
			filterPage = new FormPage( tabFolder,
					FormPage.FULL_FUNCTION,
					new FilterHandleProvider( ),
					true);
			tabItem = new TabItem( tabFolder, SWT.NONE );
			tabItem.setText( Messages.getString( "TablePageGenerator.TabItem.Filters" ) ); //$NON-NLS-1$
			tabItem.setControl( filterPage );
		}
		filterPage.setInput( input );
	}
}