/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.engine.internal.document;

import java.util.List;

import org.eclipse.birt.report.engine.api.IReportDocument;
import org.eclipse.birt.report.engine.api.impl.ReportDocumentConstants;
import org.eclipse.birt.report.engine.emitter.IContentEmitter;
import org.eclipse.birt.report.engine.executor.ExecutionContext;
import org.eclipse.birt.report.engine.internal.document.v1.ReportContentLoaderV1;
import org.eclipse.birt.report.engine.internal.document.v2.ReportContentLoaderV2;
import org.eclipse.birt.report.engine.internal.document.v3.ReportContentLoaderV3;

/**
 * content loader support mutiple version report documents.
 * 
 */
public class ReportContentLoader implements IReportContentLoader
{

	IReportContentLoader loader;

	public ReportContentLoader( ExecutionContext context )
	{
		IReportDocument document = context.getReportDocument( );
		if ( ReportDocumentConstants.REPORT_DOCUMENT_VERSION_1_0_0
				.equals( document.getVersion( ) ) )
		{
			loader = new ReportContentLoaderV1( context );
		}
		else if (ReportDocumentConstants.REPORT_DOCUMENT_VERSION_1_2_1
				.equals( document.getVersion( ) ) )
		{
			loader = new ReportContentLoaderV2( context );
		}
		else
		{
			loader = new ReportContentLoaderV3( context );
		}
	}

	public void loadPage( long pageNumber, int paginationType,
			IContentEmitter emitter )
	{
		loader.loadPage( pageNumber, paginationType, emitter );
	}

	public void loadPageRange( List pageList, int paginationType,
			IContentEmitter emitter )
	{
		loader.loadPageRange( pageList, paginationType, emitter );
	}

	public void loadReportlet( long offset, IContentEmitter emitter )
	{
		loader.loadReportlet( offset, emitter );
	}
}
