/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.engine.internal.document.v3;

import java.util.List;
import java.util.logging.Logger;

import org.eclipse.birt.report.engine.emitter.IContentEmitter;
import org.eclipse.birt.report.engine.executor.ExecutionContext;
import org.eclipse.birt.report.engine.executor.IReportExecutor;
import org.eclipse.birt.report.engine.internal.document.IReportContentLoader;
import org.eclipse.birt.report.engine.internal.executor.doc.ReportPageReader;
import org.eclipse.birt.report.engine.internal.executor.doc.ReportletReader;
import org.eclipse.birt.report.engine.internal.executor.l18n.LocalizedReportExecutor;

public class ReportContentLoaderV3 implements IReportContentLoader
{

	protected static Logger logger = Logger
			.getLogger( IReportContentLoader.class.getName( ) );

	protected ExecutionContext context;

	public ReportContentLoaderV3( ExecutionContext context )
	{
		this.context = context;
	}

	/**
	 * load the page from the content stream and output it to the emitter
	 * 
	 * @param pageNumber
	 * @param emitter
	 */
	public void loadPage( long pageNumber, int paginationType,
			IContentEmitter emitter )
	{
		ReportPageReader reader = new ReportPageReader( context, pageNumber,
				paginationType );
		LocalizedReportExecutor i18nReader = new LocalizedReportExecutor(
				context, reader ); 
		i18nReader.execute( null, emitter);
	}
	
	public void loadPageRange( List pageList, int paginationType, IContentEmitter emitter )
	{
		IReportExecutor reader = new ReportPageReader( context, pageList,
				paginationType );
		LocalizedReportExecutor i18nReader = new LocalizedReportExecutor(
				context, reader ); 
		i18nReader.execute( null, emitter);
	}
	
	public void loadReportlet( long offset, IContentEmitter emitter )
	{
		ReportletReader reader = new ReportletReader(context, offset);
		LocalizedReportExecutor i18nReader = new LocalizedReportExecutor(
				context, reader ); 
		i18nReader.execute( null, emitter);
	}
}