/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.engine.presentation;

import org.eclipse.birt.report.engine.api.IPageHandler;
import org.eclipse.birt.report.engine.content.IPageContent;
import org.eclipse.birt.report.engine.content.IReportContent;
import org.eclipse.birt.report.engine.emitter.IContentEmitter;
import org.eclipse.birt.report.engine.executor.ExecutionContext;
import org.eclipse.birt.report.engine.executor.IReportExecutor;
import org.eclipse.birt.report.engine.executor.ReportExecutor;
import org.eclipse.birt.report.engine.ir.MasterPageDesign;
import org.eclipse.birt.report.engine.ir.PageSetupDesign;
import org.eclipse.birt.report.engine.layout.ILayoutPageHandler;

public class HTMLPaginationBuilder implements IPaginationBuilder
{

	protected ExecutionContext context;

	protected HTMLPaginationEmitter layoutEmitter;

	protected IPageHandler pageHandler;

	protected IContentEmitter outputEmitter;

	public HTMLPaginationBuilder( ExecutionContext context )
	{
		this.context = context;
		layoutEmitter = new HTMLPaginationEmitter( this );
	}

	public IContentEmitter getInputEmitter( )
	{
		return layoutEmitter;
	}

	public void setOutputEmitter( IContentEmitter emitter )
	{
		layoutEmitter.setOutputEmitter( emitter );
	}

	public void setLayoutPageHandler( ILayoutPageHandler pageHandler )
	{
		layoutEmitter.setPageHandler( pageHandler );
	}

	public IPageHint getPageHint( )
	{
		return layoutEmitter.getPageHint( );
	}

	public IPageContent buildPageContent( long pageNumber, String masterPage )
	{
		IReportExecutor executor = context.getExecutor( );
		IReportContent reportContent = context.getReportContent( );
		// we support only one page master now.

		PageSetupDesign pageSetup = reportContent.getDesign( ).getPageSetup( );
		MasterPageDesign pageDesign = null;
		if ( masterPage != null )
		{
			pageDesign = pageSetup.findMasterPage( masterPage );
		}
		if ( pageDesign == null )
		{
			pageDesign = pageSetup.getMasterPage( 0 );
		}
		if ( pageDesign != null )
		{
			return ( (ReportExecutor) executor ).executeMasterPage(
					(int) pageNumber, pageDesign );
		}
		return null;
	}
}
