/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.engine.presentation;

import org.eclipse.birt.report.engine.content.IContent;
import org.eclipse.birt.report.engine.content.IReportContent;
import org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter;
import org.eclipse.birt.report.engine.emitter.ContentEmitterUtil;
import org.eclipse.birt.report.engine.emitter.IContentEmitter;
import org.eclipse.birt.report.engine.emitter.IEmitterServices;
import org.eclipse.birt.report.engine.executor.ExecutionContext;

public class LocalizedEmitter extends ContentEmitterAdapter
{
	private ExecutionContext context;
	
	protected LocalizedContentVisitor l18nVisitor;
	protected IContentEmitter emitter;

	public LocalizedEmitter( ExecutionContext context )
	{
		this( context, null );
	}

	public LocalizedEmitter( ExecutionContext context, IContentEmitter emitter )
	{
		this.emitter = emitter;
		this.context = context;
	}

	public String getOutputFormat( )
	{
		if ( emitter != null )
		{
			return emitter.getOutputFormat( );
		}
		return null;
	}
	
	public void initialize( IEmitterServices services )
	{
		super.initialize( services );
		if (emitter != null)
		{
			emitter.initialize( services );
		}
	}

	public void start( IReportContent reportContent )
	{
		super.start( reportContent );
		l18nVisitor = new LocalizedContentVisitor( context );
		if (emitter != null)
		{
			emitter.start( reportContent );
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.engine.emitter.IContentEmitter#end(org.eclipse.birt.report.engine.content.IReportContent)
	 */
	public void end( IReportContent report )
	{
		super.end( report );
		if (emitter != null)
		{
			emitter.end( report );
		}
	}

	public void startContent(IContent content)
	{
		content = l18nVisitor.localize( content );
		if (emitter != null)
		{
			ContentEmitterUtil.startContent(content, emitter);
		}
	}
	
	public void endContent (IContent content)
	{
		if (emitter != null)
		{
			ContentEmitterUtil.endContent(content, emitter);
		}
	}
}
