/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.model.script;

import org.eclipse.birt.report.model.api.DimensionHandle;

/**
 * Defines a set of methods that support a number of dimension property type
 * functions.
 *  
 */

public class Dimension
{

	/**
	 * The value handle of the dimension property type.
	 */

	DimensionHandle dimensionHandle;

	/**
	 * Constructs the dimension object with the value handle of the dimension
	 * property type.
	 * 
	 * @param dimensionHandle
	 *            the value handle of the dimension property type
	 */

	Dimension( DimensionHandle dimensionHandle )
	{
		this.dimensionHandle = dimensionHandle;
	}

	/**
	 * Returns the CSS-compatible dimension value. This is one of the CSS
	 * constants, or a value in the CSS dimension format.
	 * 
	 * @return the CSS-compatible dimension value
	 */

	public String getValue( )
	{
		return dimensionHandle.getStringValue( );
	}

	/**
	 * Gets the measure portion of a dimension, or null if the dimension is
	 * defined using a constant.
	 * 
	 * @return the measure portion of a dimension, or null if the dimension is
	 *         defined using a constant
	 */

	public double getMeasure( )
	{
		return dimensionHandle.getMeasure( );
	}

	/**
	 * Gets the units portion of a dimension, or null if the dimension is
	 * defined using a constant.
	 * 
	 * @return the units portion of a dimension, or null if the dimension is
	 *         defined using a constant
	 */

	public String getUnits( )
	{
		return dimensionHandle.getUnits( );
	}
}