/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.model.script;

import org.eclipse.birt.report.model.api.elements.structures.ConfigVariable;
import org.mozilla.javascript.ScriptableObject;

/**
 * Represents the scriptable object for <code>ConfigVariable</code>.
 * 
 * @see org.eclipse.birt.report.model.api.elements.structures.ConfigVariable
 */

public class NativeConfigVariable extends ScriptableObject
{

	/**
	 * Class name in JavaScript.
	 */

	public static final String CLASS_NAME = "ConfigVariable"; //$NON-NLS-1$

	/**
	 * The cached config variable.
	 */

	private ConfigVariable variable = null;

    /**
     * The zero-argument constructor used by Rhino runtime to create instances
     */

	public NativeConfigVariable( )
	{}

     /**
     * JavaScript function that is the constructor .
     * 
     * @param variable a ConfigVariable instance
     */
    
    public void jsConstructor( Object variable )
    {
        assert (variable instanceof ConfigVariable );
        this.variable = (ConfigVariable) variable;
    }
    
    /**
     * Returns the config variable name.
     *  
     * @return name of the config variable.
     */
    
    public String jsGet_name()
    {
    	return this.variable.getName();
    }
    
    
    /**
     * Returns the config variable value.
     *  
     * @return value of the config variable.
     */
    
    public String jsGet_value()
    {
    	return this.variable.getValue();
    }
    
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.mozilla.javascript.Scriptable#getClassName()
	 */
    
	public String getClassName( )
	{
		return CLASS_NAME;
	}

	
}